﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class GameController : MonoBehaviour
{
	public int RequiredBeatsToWin = -1;
	public float TimeToSurviveTotal = -1f;
	public float TimeToSurviveWithoutMissingAPulse = -1f;
	public float RequiredBeatsWithoutInterruption = -1f;

	public int BeatsTotal = 0;
	public int BeatsWithoutInterruption = 0;
	public float TimeSurvivedTotal;
	public float TimeSurvivedWithoutInterruption;

	public float TimeToDieAfterLastPulse = 3;
	public int PulsesToMissInSequence = -1;
	public int PulsesToMiss = -1;

	public float PassedTimeSincePulse = 0;
	public int PulsesMissedInSequence = 0;
	public int PulsesMissedTotal = 0;

	public string NextScene = "";
	public string GameOverScene = "";

	private void Start()
	{
		StatusManager.Instance.AntiPulse += AntiPulse;
		StatusManager.Instance.Pulse += Pulse;
	}

	private void CheckForVictory()
	{
		if( ((RequiredBeatsToWin > -1 ==  BeatsTotal >= RequiredBeatsToWin) || RequiredBeatsToWin <= -1) &&
			((RequiredBeatsWithoutInterruption > -1 == BeatsWithoutInterruption >= RequiredBeatsWithoutInterruption) || RequiredBeatsWithoutInterruption <= -1) &&
			((TimeToSurviveTotal > -1 == TimeSurvivedTotal >= TimeToSurviveTotal) || TimeToSurviveTotal <= -1) &&
			((TimeToSurviveWithoutMissingAPulse > -1 == TimeSurvivedWithoutInterruption >= TimeToSurviveWithoutMissingAPulse) || TimeToSurviveWithoutMissingAPulse <= -1) &&
			!(RequiredBeatsToWin <= -1 && RequiredBeatsWithoutInterruption <= -1 && TimeToSurviveTotal <= -1 && TimeToSurviveWithoutMissingAPulse <= -1))
		{
			Victory();
		}
	}

	private void CheckForLoss()
	{
		if( (TimeToDieAfterLastPulse > -1 && PassedTimeSincePulse >= TimeToDieAfterLastPulse) ||
			(PulsesToMissInSequence > -1 && PulsesMissedInSequence > PulsesToMissInSequence) ||
			(PulsesToMiss > -1 && PulsesMissedTotal > PulsesToMiss))
		{
			GameOver();
		}
	}

	private void Victory()
	{
		print("You won!");
		if(!string.IsNullOrEmpty(NextScene))
		{
			//LOAD LEVEL
			UnityEngine.SceneManagement.SceneManager.LoadScene(NextScene);
		}
	}

	private void GameOver()
	{
		print("You're D.E.A.D.");
		if (!string.IsNullOrEmpty(GameOverScene))
		{
			//LOAD LEVEL
			UnityEngine.SceneManagement.SceneManager.LoadScene(GameOverScene);
		}
	}

	private void Update()
	{
		CheckForVictory();
		CheckForLoss();


		TimeSurvivedTotal += Time.deltaTime;
		PassedTimeSincePulse += Time.deltaTime;
		if (StatusManager.Instance.CheckMachines())
		{
			TimeSurvivedWithoutInterruption += Time.deltaTime;
		}
	}

	private void Pulse()
	{
		BeatsTotal += 1;
		BeatsWithoutInterruption += 1;

		PulsesMissedInSequence = 0;
		PassedTimeSincePulse = 0;
	}

	private void AntiPulse()
	{
		BeatsWithoutInterruption = 0;
		TimeSurvivedWithoutInterruption = 0;

		PulsesMissedTotal += 1;
		PulsesMissedInSequence += 1;
	}
}
