﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class GenericIObjectConsumer : IInteractionObject
{
	public float timeLeft = 0;
	public float maxTime = 5.0f;
	public SpriteRenderer Visualizer;
	public Ressource RequiredRessource = Ressource.None;

	public override void Interact(PlayerController sender)
	{
		var res = sender.GetComponentInChildren<RessourceScript>();
		if(res == null || res.Ressource != RequiredRessource)
		{
			return;
		}
		res.Remove();
		timeLeft = maxTime;
	}

	public int StartTime = -1;

	public GameObject Rotator;

	public Ressource[] PossibleRessources;

	public void Start()
	{
		if (StartTime < 0)
		{
			timeLeft = maxTime;
		}
		if (RequiredRessource == Ressource.None)
		{
			ChangeRequiredRessource(RandomRessource());
		}
		else
		{
			ChangeRequiredRessource(RequiredRessource);
		}
	}

	public void ChangeRequiredRessource(Ressource r)
	{
		var rend = GetComponent<SpriteRenderer>();
		RequiredRessource = r;
		switch (r)
		{
			case Ressource.BlueBalls:
				rend.sprite = Resources.Load<Sprite>("Sprites/Maschine3");
				break;
			case Ressource.RedBalls:
				rend.sprite = Resources.Load<Sprite>("Sprites/Maschine2");
				break;
			case Ressource.YellowBalls:
				rend.sprite = Resources.Load<Sprite>("Sprites/Maschine1");
				break;
			default:
				break;
		}
	}

	public Ressource RandomRessource()
	{
		return PossibleRessources[Random.Range(0, PossibleRessources.Length)];
	}

	public override bool Active()
	{
		if (timeLeft > 0)
			return true;
		return false;
	}

	// Update is called once per frame
	void Update()
	{
		timeLeft -= Time.deltaTime;
		if (timeLeft > 0)
		{
			Rotator.transform.localRotation = Quaternion.Euler(0, 0, -125f + (1.0f - (timeLeft / maxTime)) * 250f);
		}
		else
		{
			Rotator.transform.localRotation = Quaternion.Euler(0, 0, +125f);
		}

		if (timeLeft < 0)
			timeLeft = 0;
	}
}
