﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

public class PlayerController : MonoBehaviour
{
    public string playerId = "";
    public float speed = 0.1f;
    public int jumpstrength = 20;

    private bool onLadder = false;
    private bool ladderSafe = false;
    private float defaultGravityScale;

    private Rigidbody2D rb;// GetComponent<Rigidbody2D>();
    private Collider2D col;
    private Animator animator;
    public bool onGround = false;
    private bool bypassFloor = false;
    private RessourceScript rs;

	private IInteractionObject interactable = null;

    // Use this for initialization
    void Start ()
    {
        rb = GetComponent<Rigidbody2D>();
        col = GetComponent<Collider2D>();
        animator = GetComponent<Animator>();
    }

	void Update ()
	{
        float horizontal = Input.GetAxis(string.Concat("Horizontal", playerId));
	    float vertical = Input.GetAxis(string.Concat("Vertical", playerId));

	    rs = GetComponentInChildren<RessourceScript>();

	    if (horizontal > 0)
            transform.localScale = new Vector3(1,1,1);
        else if (horizontal < 0)
            transform.localScale = new Vector3(-1,1,1);

		transform.Translate(horizontal * Time.deltaTime * speed, onLadder ? vertical * speed * Time.deltaTime : 0, 0);
		
	    if (onGround && Input.GetButtonDown(String.Concat("Jump",playerId)))
	    {
	        rb.AddForce(new Vector2(0,jumpstrength));
            animator.SetTrigger("jump");
	        onGround = false;

	        if (onLadder)
	            rb.gravityScale = defaultGravityScale;
	    }
		if(interactable != null && Input.GetButtonDown(String.Concat("action",playerId)))
		{
			interactable.Interact(this);
		}

            
        animator.SetBool("climbing", Math.Abs(vertical) > 0);
        animator.SetBool("walking", Math.Abs(horizontal) > 0);

	    if (rs != null && Input.GetButtonDown(String.Concat("drop", playerId)))
	    {
            print("drop");
            rs.GetComponent<Rigidbody2D>().gravityScale = 1;
            foreach (CircleCollider2D col in rs.GetComponents<CircleCollider2D>())
                if (col.radius == 0.5f)
                    col.isTrigger = false;
            rs.transform.SetParent(null);
        }

        if (Input.GetKeyDown(KeyCode.LeftControl) && Input.GetKeyDown(KeyCode.UpArrow) && Input.GetKeyDown(KeyCode.Keypad0))
            SceneManager.LoadScene("heart");

        if (Input.GetButtonDown("Cancel"))
            SceneManager.LoadScene("menu");
	}

    public void FixedUpdate()
    {
        if (rs != null)
        {
            rs.gameObject.transform.position = transform.position + new Vector3(0, 1.5f, 1);
        }
    }

    public void OnCollisionEnter2D(Collision2D other)
    {
        if (other.gameObject.CompareTag("ground"))
            onGround = true;
    }

    public void OnTriggerEnter2D(Collider2D other)
    {
		if(other.gameObject.CompareTag("InteractionObject") || other.gameObject.CompareTag("Ressource"))
		{
			interactable = other.GetComponent<IInteractionObject>();
		}
    }

	public void OnTriggerExit2D(Collider2D other)
	{
		if (other.gameObject.CompareTag("InteractionObject") || other.gameObject.CompareTag("Ressource"))
		{
			interactable = null;
		}
	}

	public void SetLadder(bool b1)
    {
        onLadder = b1;
        if (b1 && !ladderSafe)
        {
			print("ladder enter");
            ladderSafe = true;
            GetComponent<Animator>().SetBool("onladder",true);
            defaultGravityScale = rb.gravityScale;
            rb.gravityScale = 0;
        }
        else
		{
			print("ladder exit");
			GetComponent<Animator>().SetBool("onladder", false);
            rb.gravityScale = defaultGravityScale;
            ladderSafe = false;
            //col.isTrigger = false;      
        }
    }

}
