﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PulsingSpotlight : MonoBehaviour
{
	public enum State { Normal, SinglePulse, FadeToDeath, Dead };

	public Color cColorNormal1;
	public Color cColorNormal2;
	public float cMaxIntensityNormal;
	public float cMinIntensityNormal;
	public float cRegenerationNormal;

	public Color cColorLastPulse1;
	public Color cColorLastPulse2;
	public float cMaxIntensityLastPulse;
	public float cMinIntensityLastPulse;
	public float cRegenerationLastPulse;

	public Color cColorDeath1;
	public Color cColorDeath2;
	public float cMaxIntensityDeath;
	public float cMinIntensityDeath;
	public float cRegenerationDeath;

	public float cTimePassed = 0;
	public float cRegenerationTime;

	public State Status;

	// Use this for initialization
	void Start ()
	{
		_light = GetComponent<Light>();

		cColorNormal1 = cColorNormal2 = _light.color;
		cColorLastPulse2 = cColorLastPulse1;
		cColorDeath1 = cColorLastPulse2;

		cRegenerationNormal = StatusManager.Instance.RecalculatePulseFrequencyInSeconds();
	}

	Light _light;

	public void Revive()
	{
		cTimePassed = 0;
		Status = State.Normal;
	}

	// Update is called once per frame
	void Update ()
	{
		if (Status == State.Dead)
			return;

		cTimePassed += Time.deltaTime;
		float maxIntensity;
		float minIntensity;
		float regeneration;
		Color c1;
		Color c2;
		switch(Status)
		{
			case State.FadeToDeath:
				maxIntensity = cMaxIntensityDeath;
				minIntensity = cMinIntensityDeath;
				regeneration = cRegenerationDeath;
				c1 = cColorDeath1;
				c2 = cColorDeath2;
				break;
			case State.SinglePulse:
				maxIntensity = cMaxIntensityLastPulse;
				minIntensity = cMinIntensityLastPulse;
				regeneration = cRegenerationLastPulse;
				c1 = cColorLastPulse1;
				c2 = cColorLastPulse2;
				break;
			default:
				maxIntensity = cMaxIntensityNormal;
				minIntensity = cMinIntensityNormal;
				regeneration = cRegenerationNormal;
				c1 = cColorNormal1;
				c2 = cColorNormal2;
				break;
		}
		_light.intensity = Mathf.Lerp(maxIntensity, minIntensity, cTimePassed / regeneration);
		_light.color = Color.Lerp(c1, c2, cTimePassed / regeneration);
		if (!StatusManager.Instance.CheckMachines())
		{
			if (Status == State.Normal)
				Status = State.SinglePulse;
			else if (Status == State.SinglePulse && cTimePassed > regeneration)
				Status = State.FadeToDeath;
			else if(Status == State.FadeToDeath && cTimePassed > regeneration)
			{
				Status = State.Dead;
				return;
			}
		}
		else
		{
			Status = State.Normal;
		}
		if (cTimePassed < regeneration)
		{
			return;
		}
		cTimePassed = 0;
		_light.intensity = maxIntensity;
	}
}
