﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class StatusManager : MonoBehaviour
{
	private static StatusManager _this;
	public static StatusManager Instance
	{
		get
		{
			return _this;
		}
	}

	public int ReqPulse = 70;
	public int PulseTolerance = 10;

	public int ReqPressure = 120;

	public int UpdateFrequencyInSeconds = 2;
	private float PulseFrequencyInSeconds = 0.5f;
	public int PulseFrequencyPerSecond = 120;
    public int SecondsPerYear = 15;

    public int startAge = 20;

	public List<IInteractionObject> interObjects;

    private int age = 0;
    private float angina = 0;

	private float timePassedChecking = 0;
	private float timePassedPulsing = 0;
    private float timePassedAge = 0;

	public float timeThreSholdAfterPulse = 3;
	private float timePassedForThreshold = 0;

	public int beats = 0;

	public AudioSource AS;

	public void AddMachine(IInteractionObject iObj)
	{
		interObjects.Add(iObj);
	}

	private void Awake()
	{
		if(_this == null)
		{
			_this = this;
		}
		else if(_this != this)
		{
			Destroy(_this);
		}
		Pulse += StatusManager_Pulse;
		AntiPulse += StatusManager_AntiPulse;
		interObjects = new List<IInteractionObject>();
		RecalculatePulseFrequencyInSeconds();
        
		var tmp = GameObject.FindGameObjectsWithTag("InteractionObject");
		foreach(GameObject obj in tmp)
		{
				interObjects.Add(obj.GetComponent<IInteractionObject>());
		}
	    age = startAge;
	}

	private void StatusManager_AntiPulse()
	{
	}

	public float RecalculatePulseFrequencyInSeconds()
	{
		PulseFrequencyInSeconds = 60.0f / PulseFrequencyPerSecond;
		return PulseFrequencyInSeconds;
	}

	private void StatusManager_Pulse()
	{
		beats += 1;
		StopLongBeep();
		timePassedForThreshold = 0;
	}

	void Update()
	{
		timePassedChecking += Time.deltaTime;
		timePassedPulsing += Time.deltaTime;
	    timePassedAge += Time.deltaTime;
		timePassedForThreshold += Time.deltaTime;

		if(timePassedChecking > UpdateFrequencyInSeconds)
		{
			CheckRequirement();
			timePassedChecking = 0;
			beats = 0;
		}
		if(timePassedPulsing > PulseFrequencyInSeconds)
		{
			PulseUpdate();
			timePassedPulsing = 0;
		}

	    if (timePassedAge > SecondsPerYear)
	    {
	        AgeUpdate();
	        timePassedAge = 0;
	    }

		if(timePassedForThreshold > timeThreSholdAfterPulse)
		{
			StartLongBeep();
		}
	}

	void StartLongBeep()
	{
		if (!AS.isPlaying)
		{
			if(!AS.enabled)
			{
				AS.enabled = true;
			}
			AS.Play();
		}
	}

	void StopLongBeep()
	{
		AS.Stop();
	}

	private bool CheckRequirement()
	{
		int curPulse = (int)((double)UpdateFrequencyInSeconds * (60.0 / (double)UpdateFrequencyInSeconds)) * beats;
		if(curPulse >= ReqPulse - PulseTolerance|| curPulse <= ReqPulse + PulseTolerance)
		{
			//You succeed
			return true;
		}
		//You loose
		return false;
	}

	public bool CheckMachines()
	{
		foreach(var iObj in interObjects)
		{
			if(!iObj.Active())
			{
				return false;
			}
		}
		return true;
	}

	private void PulseUpdate()
	{
		if(CheckMachines())
		{
			Pulse.Invoke();
		}
		else
		{
			AntiPulse.Invoke();
		}
	}

    private void AgeUpdate()
    {
        CalculateAnginaPectoris();
    }

    private void CalculateAnginaPectoris()
    {
        //todo
    }

	//Pulse Event
	public delegate void PulseHandler();
	public event PulseHandler Pulse;

	//AntiPulse Event
	public delegate void AntiPulseHandler();
	public event AntiPulseHandler AntiPulse;
}
